//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : Controller
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This Controller class is in charge of constructing controls for the view
// (the view class calls these methods and places them appropriately) and
// connecting them to the model (calling model methods when the control is
// activated, and passing along relevant information).
//
//   Typically, a control pushes data to the model. Some controls also allow
// the model to pull data from them (see the instance variables and the
// pullPress method in the Model class).
//
//   Note that "no access modifier" means that the method is package
// friendly: this means the member is public to all other classes in
// the ControllerDemo package, but private elsewhere.
//
// Future Plans   : More Comments
//
// Program History:
//   10/20/01: R. Pattis - Operational for 15-100/15-111
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////



package controllerDemo;


//Listener interfaces
import java.awt.event.ActionListener;
import javax.swing.event.ChangeListener;
import java.awt.event.FocusListener;
import java.awt.event.MouseListener;
import java.awt.event.KeyListener;

//Adapters (for complicated listeners)
import java.awt.event.FocusAdapter;
import java.awt.event.MouseAdapter;
import java.awt.event.KeyAdapter;

//Event parameters for listeners 
import java.awt.event.ActionEvent;
import javax.swing.event.ChangeEvent;
import java.awt.event.FocusEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

//Controller Components
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

//Misc (see mouse listener)
import java.awt.Cursor;
import java.awt.Dimension;


public class Controller
{

  //Store cursors to use in methods in getMouseListener
  public Controller()
  {
    //Do this once, at construction time, rather than each
    //  time a mouse enter/exit event occurs.
  }



  //Refer to the Model
	public void addModel(Model m)
	{model = m;}

	
  
  //Build/Return a generic Button with a label
  JButton getButton(String label)
  {
	  final JButton b = new JButton(label);
	  
	  b.addActionListener(new ActionListener() {
		  public void actionPerformed(ActionEvent e)
		  {
		    System.out.println("Debug-Controller: " + "Button pressed: " + b.getText() );
		    //System.out.println("  Action-Event = " + e);
		    model.buttonPress(b.getText());
      }});
	  
	  return b;
  }
  
  

  //Build/Return a generic Button with a label that switches
  JButton getSwitchingButton(final String[] labels)
  {
   final JButton b = new JButton(labels[0]);

   b.addActionListener(
     new ActionListener() {
       private int currentState = 0;
    
       public void actionPerformed(ActionEvent e)
       {
         model.buttonPress(b.getText());
         currentState = (currentState+1)%labels.length;
         b.setText(labels[currentState]);
      }});
	  
    return b;
  }
  


  //Build/Return a button for pulling all data
  JButton getPullButton(String label)
  {
	  final JButton b = new JButton(label);
	  
	  b.addActionListener(new ActionListener() {
		  public void actionPerformed(ActionEvent e)
		  {
		    System.out.println("Debug-Controller: " + "Pull Button pressed");
		    //System.out.println("  Action-Event = " + e);
		    model.pullPress();
      }});
	  
	  return b;
  }
  
  

  //Build/Return a text field for pushing (and pulling) text data
  //Data is pushed when the user presses Enter after entering data
  //  or the users moves the cursor somewhere else (and performs
  //  an action there)
  //Since all text fields call the same model method, pass along the
  //  id of the textField, so the model can recognize which text field
  //  it is (and act appropriately)
  JTextField getTextField(final String id, String contents, int displaySize)
  {
	  final JTextField tf = new JTextField(contents,displaySize);
	  
	  tf.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e)
	    {
		    System.out.println("Debug-Controller: " + "Text field "+id+" activated: " +
		                        tf.getText());
		    //System.out.println("  Action-Event = " + e);
	      model.textFieldEntered(id, tf.getText());
	    }
	  });
	  
	  tf.addFocusListener(new FocusAdapter() {
	    public void focusLost(FocusEvent e)
	    {
		    System.out.println("Debug-Controller: " + "Text field "+id+" loses focus: " +
		                       tf.getText());
		    //System.out.println("  Focus-Event = " + e);
	      model.textFieldEntered(id, tf.getText());
	    }
	  });
	  
	  return tf;
	} 
	
	
	  
  //Build/Return a panel with various radio buttons (their labels are
  //  specified via the strings below).
  //Data is pushed when the user presses clicks a button
  //Note that the instance variable decisionPanel is set for pulling data
  RadioButtonPanel getDecisionPanel(String[] labels)
  {
	 
	  return decisionPanel = 
	    new RadioButtonPanel(labels,
	     new ActionListener() {
         public void actionPerformed(ActionEvent e)
         {model.decisionPanelUpdate(decisionPanel.getSelectedValue());}}
	    );
	} 
	  
  

  //Build/Return a panel with various radio buttons (their labels are
  //  specified via the strings below).
  //Data is pushed when the user presses clicks a button
  //Note that the instance variable weatherPanel is set for pulling data
  RadioButtonPanel getWeatherPanel(String[] labels)
  {
	 
	  return weatherPanel = 
	    new RadioButtonPanel(labels,
	                         new ActionListener() {
                             public void actionPerformed(ActionEvent e)
                             {model.weatherPanelUpdate(weatherPanel.getSelectedValue());}}
	                        );
	} 
	  
  

  //Build/Return a label and a slider from 0-100%
  //Data is pushed when the user changes the setting; one big change can
  //  push a series of data values
  //Note that the instance variable rainPanel is set for pulling data
  PercentSliderPanel getRainSlider(String label)
  {
	 
	  return rainPanel = 
	    new PercentSliderPanel(label,
	                           new ChangeListener() {
                               public void stateChanged(ChangeEvent e)
                              {model.rainPanelUpdate(rainPanel.getSelectedValue());}}
	                          );
	} 
	  
  	  
	  
  //Build/Return a mouse listener that will be added to some JPanel
  //Data is pushed when the user clicks the mouse
  //The representation of the mouse is changed when it enters/exits
  //  the JPanel
  MouseListener getMouseListener(final JPanel source)
  {
    return
      new MouseAdapter() {
      
        Cursor inPane   = new Cursor(Cursor.HAND_CURSOR);
        Cursor standard = new Cursor(Cursor.DEFAULT_CURSOR);
        
        public void mouseClicked(MouseEvent e) 
        {
		      System.out.println("Debug-Controller: Mouse clicked ("
		                         +e.getClickCount()+") at ("+e.getX()+","
		                         +e.getY()+")");
		      //System.out.println("  Mouse-Event = " + e);
          model.mouseClick(e.getX(),e.getY(),e.getClickCount());
        }
        
        public void mouseEntered(MouseEvent e)
        {
		      //System.out.println("Debug-Controller: Mouse entered");
		      //System.out.println("  Mouse-Event = " + e);
          source.setCursor(inPane);
        }
         
        public void mouseExited(MouseEvent e) 
        {
		      //System.out.println("Debug-Controller: Mouse exited");
		      //System.out.println("  Mouse-Event = " + e);
          source.setCursor(standard);
        }

      };
  }



  //Build/Return a key listener that will be added to some JFrame
  //  (not JPanel): a JFrame can have at most one key listener
  KeyListener getKeyListener()
  {
    return

      new KeyAdapter() {
				public void keyTyped(KeyEvent evt)
				{
		      System.out.println("Debug-Controller: Key pressed ("
		                         +evt.getKeyChar()+")");
				  model.keyTyped(evt.getKeyChar());
				}};
	}
  

  //instance variables (for model, pulling information)
  Model model;
  
  RadioButtonPanel decisionPanel;
  RadioButtonPanel weatherPanel;
  PercentSliderPanel rainPanel;
}


